/*
 * Copyright (c) 2018 Gleb Fotengauer-Malinovskiy.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <openssl/engine.h>
#include <openssl/conf.h>

#include "openbsd-compat/openssl-compat.h"

static void
init_algorithms() {
	static int algorithms_init = -1;

	if (algorithms_init < 0) {
		OpenSSL_add_all_algorithms();
		ERR_load_crypto_strings();
		algorithms_init = 1;
	}
}

static const EVP_MD*
get_gost_md_by_nid(int nid)
{
	init_algorithms();

	const EVP_MD* digest;
	if (!(digest = EVP_get_digestbynid(nid)))
	    return NULL;

	return digest;
}

static const EVP_CIPHER*
get_gost_cipher_by_nid(int nid)
{
	init_algorithms();

	const EVP_CIPHER *cipher;
	if (!(cipher = EVP_get_cipherbynid(nid)))
		return NULL;

	return cipher;
}

const EVP_MD*
EVP_streebog_256()
{
	return get_gost_md_by_nid(NID_id_GostR3411_2012_256);
}

const EVP_MD*
EVP_streebog_512()
{
	return get_gost_md_by_nid(NID_id_GostR3411_2012_512);
}

const EVP_MD*
EVP_grasshopper_omac()
{
	return get_gost_md_by_nid(NID_grasshopper_mac);
}

const EVP_CIPHER*
EVP_magma_cbc()
{
	return get_gost_cipher_by_nid(NID_magma_cbc);
}

const EVP_CIPHER*
EVP_magma_ctr()
{
	return get_gost_cipher_by_nid(NID_magma_ctr);
}

const EVP_CIPHER*
EVP_grasshopper_cbc()
{
	return get_gost_cipher_by_nid(NID_grasshopper_cbc);
}

const EVP_CIPHER*
EVP_grasshopper_ctr()
{
	return get_gost_cipher_by_nid(NID_grasshopper_ctr);
}
